<?php

namespace Bitrix\KMsending\Template;

use Bitrix\Main\Localization\Loc;
use CEventMessage;
use COption;
use CSite;
use Bitrix\KMsending\Config;
use Bitrix\KMsending\Exceptions\KMsendingException;

class Service
{
    private const USER_PREFIX = 'KMSENDING_';
    private const ADMIN_PREFIX = 'KMSENDING_ADMIN_';

    private $config;

    public function __construct()
    {
        $this->config = new Config\Service();
    }

    /**
     * @param string $mailEvent
     * @param string $siteId
     *
     * @return array
     * @throws KMsendingException
     */
    public function getUserTemplate(string $mailEvent, $siteId): array
    {
        $templateName = self::USER_PREFIX . $mailEvent;
        if (!$template = $this->getTemplateByName($templateName, $siteId)) {
            throw new KMsendingException(Loc::getMessage('KMSENDING_TEMPLATE_NOT_FOUND_ERROR',
                ['#TEMPLATE#' => $templateName]));
        }
        return $template;
    }

    /**
     * @param string $mailEvent
     * @param string $siteId
     *
     * @return array
     * @throws KMsendingException
     */
    public function getAdminTemplate(string $mailEvent, $siteId): array
    {
        $templateName = self::ADMIN_PREFIX . $mailEvent;
        if (!$template = $this->getTemplateByName($templateName, $siteId)) {
            throw new KMsendingException(Loc::getMessage('KMSENDING_TEMPLATE_NOT_FOUND_ERROR',
                ['#TEMPLATE#' => $templateName]));
        }

        return $template;
    }

    /**
     * @param string $mailEvent
     * @param string $siteId
     * @param string $from
     *
     * @return array
     * @throws KMsendingException
     */
    public function getCustomSmsTemplate(string $mailEvent, $siteId, string $from = ''): array
    {
        if (!$template = $this->getTemplateByName($mailEvent, $siteId, $from)) {
            throw new KMsendingException(Loc::getMessage('KMSENDING_TEMPLATE_NOT_FOUND_ERROR',
                ['#TEMPLATE#' => $mailEvent]));
        }
        return $template;
    }

    /**
     * @param string $mailEvent
     * @param string $siteId
     * @param string $from
     *
     * @return array
     * @throws KMsendingException
     */
    public function getTemplateForSmsEvent(string $mailEvent, $siteId, string $from): array
    {
        $templateName = self::USER_PREFIX . $mailEvent;
        if (!$template = $this->getTemplateByName($templateName, $siteId, $from)) {
            throw new KMsendingException(Loc::getMessage('KMSENDING_TEMPLATE_NOT_FOUND_ERROR',
                ['#TEMPLATE#' => $templateName]));
        }

        return $template;
    }

    /**
     * @param string $mailEvent
     * @param        $siteId
     * @param string $from
     *
     * @return array
     * @throws KMsendingException
     */
    public function findTemplateForSmsEvent(string $mailEvent, $siteId, string $from): array
    {
        return $this->getTemplateByName(self::USER_PREFIX . $mailEvent, $siteId, $from);
    }

    /**
     * @param int $templateId
     *
     * @return string
     * @throws KMsendingException
     */
    public function getTemplateTextById($templateId): string
    {
        try {
            $text = \Bitrix\Main\Mail\Internal\EventMessageTable::getList([
                'select' => ['*'],
                'filter' => ['=ID' => $templateId]
            ]);

            $res = $text->fetch();

        } catch (\Exception $e) {
            throw new KMsendingException(Loc::getMessage('KMSENDING_TEMPTALE_ERROR'), $e->getCode(), $e);
        }

        if (!$text = $res['MESSAGE']) {
            throw new KMsendingException(Loc::getMessage('KMSENDING_TEMPLATE_NOT_FOUND_BY_ID',
                ['#TEMPLATE_ID#' => $templateId]));
        }
        return $res['MESSAGE'];
    }

    /**
     * @param string $pattern
     * @param string $siteId
     *
     * @return array
     * @throws KMsendingException
     */
    public function getAllSmsTemplate(string $pattern, string $siteId): array
    {
        try {
            $arMess = [];
            $pattern = !empty($pattern) ? $pattern : 'KMSENDING_%';
            $arFilter = ['TYPE_ID' => $pattern];
            if (!empty($siteId)) {
                $arFilter['SITE_ID'] = $siteId;
            }
            $rsMess = CEventMessage::GetList($by = 'id', $order = 'asc', $arFilter);
            while ($res = $rsMess->Fetch()) {
                $arMess[$res['EVENT_NAME']][] = ['ID' => $res['ID'], 'NAME' => $res['SUBJECT']];
            }
            return $arMess;
        } catch (\Exception $e) {
            throw new KMsendingException(Loc::getMessage('KMSENDING_TEMPTALE_ERROR'), $e->getCode(), $e);
        }
    }

    /**
     * �������� ����������� ������� � ������
     * �������� ������� ��-��������� � ������
     *
     * @param string $text - �������� ����� ��� ������
     *
     * @return string - ����� � ����������� ������������ ���������
     */
    public function replaceDefaultMakros(string $text): string
    {
        $res = CSite::GetByID($this->config->getSiteIdDefault());
        $arRes = $res->Fetch();

        $text = str_replace('#DEFAULT_EMAIL_FROM#', COption::GetOptionString('main', 'email_from'), $text);
        $text = str_replace('#SITE_NAME#', $arRes['SITE_NAME'], $text);
        $text = str_replace('#SERVER_NAME#', $arRes['SERVER_NAME'], $text);
        return $text;
    }

    /**
     * @param string $templateName
     * @param string $siteId
     * @param string $from
     *
     * @return array
     * @throws KMsendingException
     */
    private function getTemplateByName(string $templateName, $siteId, $from = ''): array
    {
        try {
            $arFilter = [
                'ACTIVE'  => 'Y',
                'TYPE_ID' => $templateName,
                'SITE_ID' => $siteId,
            ];

            if (!empty($from)) {
                $arFilter['FROM'] = $from;
            }

            $rsMess = CEventMessage::GetList($by = 'site_id', $order = 'desc', $arFilter);

            return ($template = $rsMess->fetch()) ? $template : [];

        } catch (\Exception $e) {
            throw new KMsendingException(Loc::getMessage('KMSENDING_TEMPLATE_ERROR'), $e->getCode(), $e);
        }
    }

    /**
     * @param string $template
     * @param array  $marcos
     *
     * @return string
     */
    public function fillInTemplateWithMacros(string $template, array $marcos): string
    {
        foreach ($marcos as $k => $value) {
            $template = str_replace('#' . $k . '#', $value, $template);
        }

        return $this->replaceDefaultMakros($template);
    }
}